// Copyright (C) 1999 Jean-Marc Valin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef AUDIO_FILE_H
#define AUDIO_FILE_H

#include "Node.h"
#include "Object.h"
#include <stdio.h>
#include "net_types.h"
#include <fstream>

/**Class for nodes that reads audio from a file*/
class AudioFile : public Node {
protected:
   /**The opened file*/
   FILE *file;

   /**The output frame buffer (type RotatingMatrix)*/
   ObjectRef output;

   /**Number of frames to cache at the output*/
   int outputBufferLength;

   /**Length of output frames*/
   int outputLength;

   /**Number of samples between start of frames*/
   int frameAdvance;

   /**outputID*/
   int outputID;

   /**inputID*/
   int inputID;

   /**The last input stream received (used to know whether it's still the same file)*/
   Stream *lastFile;

public:
   /**supported encodings*/
   typedef enum {ulaw, alaw, lin8, lin16} EncodeType;

   /**Constructor, takes the name of the node and a set of parameters*/
   AudioFile(string nodeName, ParameterSet params);

   /**Destructor*/
   ~AudioFile();

   /**Class specific initialization routine.
      Each class will call its subclass specificInitialize() method*/
   virtual void specificInitialize();

   /**Class reset routine.
      Each class will call its superclass reset() method*/
   virtual void reset();

   /**Ask for the node's output which ID (number) is output_id 
      and for the 'count' iteration */
   virtual ObjectRef getOutput(int output_id, int count); 

private:
   /**Encoding type for samples*/
   EncodeType encoding;

   /**Sample size (in bytes) either 1 or 2*/
   int itemSize;

   /**Conversion function*/
   void raw2Float (void *in, float *out, int length, EncodeType encoding);

   /**Set encoding type in constructor*/
   void setEncoding(const string &enc);

   /**Internal temporary variable used when converting sample to float*/
   char *tmpBuffer;

   /**Conversion table for A-law*/
   static short alaw2linear[256];

   /**Conversion table for U-law*/
   static short ulaw2linear[256];
protected:

   /**Default constructor, should not be used*/
   AudioFile() {throw new GeneralException("AudioFile copy constructor should not be called",__FILE__,__LINE__);}

};

#endif
